#ifndef _SPARSE_MATRIX_HPP_
#define _SPARSE_MATRIX_HPP_

#include <iostream>
#include <fstream>
#include <vector>

typedef unsigned int uint;
typedef std::vector<double> rvec;

class SparseMatrix {

  public: 

    // Empty default constructor
    SparseMatrix() {};

    // Constructor with size and #of non zero coeffs
    SparseMatrix(const int, const int);

    // Copy constructor
    SparseMatrix(const SparseMatrix&);

    // Constructor from text file
    SparseMatrix(const std::string fileName);

    // Read-only access 
    double getValue(const int i, const int j);

    // Computes infinite norm of matrix (max over rows of sum of abs of coefficients)
    double inftNorm();

    // Computes trace of matrix (i.e. sum of diagonal terms)
    double trace();

    // Matrix vector product
    void mult(const rvec&, rvec&);

    // Displays matrix in given stream, called by operator <<
    void print(std::ostream&);

    // Read and write access. If coefficent is not in list, it is added.
    double& operator()(const int, const int);

    // Uses power iteration method to compute dominating eigen value of sparse matrix

  friend SparseMatrix operator+ (const SparseMatrix&, const SparseMatrix&);
  friend rvec         operator* (const SparseMatrix&, const rvec&);

  private:

    // Pos of coeff (i,j) in _vals if non zero, -1 otherwise
    int posIndex(const int i, const int j);
 
    uint _size;  // Size of whole matrix
    uint _nNz;   // # of non zero terms

    std::vector<uint> _rows,_cols; // row and column indices of non zero terms
    rvec              _vals;       // non zero terms

};

// Calls print() method of matrix
std::ostream& operator<<(std::ostream&, SparseMatrix&);


#endif

